package net.metanotion.io.block;

import java.io.Closeable;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;

import net.metanotion.io.RandomAccessInterface;

abstract public class BlockHandle implements RandomAccessInterface, DataInput, DataOutput, Closeable {
	private BlockFile bf;
	private long id;
	private long blockLen;
	private RandomAccessInterface file;
	private long curOffset;
	private long fileOffset;

	public static void init(BlockFile bf, long bid, int page) throws IOException {
		BlockFile.pageSeek(bf.file, page);
		bf.file.writeInt(0);
		bf.file.writeInt(0);
		bf.file.writeLong(bid);
	}

	public BlockHandle(long bid, RandomAccessInterface file, long length, long fileOffset, BlockFile bf) {
		this.bf = bf;
		this.id = bid;
		this.file = file;
		this.blockLen = length;
		this.curOffset = 0;
		this.fileOffset = fileOffset;
	}

	public long getFilePointer() throws IOException { return curOffset + fileOffset; }
	public long length() throws IOException { return this.blockLen; }

	public int read() throws IOException {
		if(curOffset >= blockLen) { return -1; }
		file.seek(curOffset + fileOffset);
		curOffset++;
		return file.read();
	}
//	public int read(byte[] b) throws IOException;
//	public int read(byte[] b, int off, int len) throws IOException;

	public void seek(long pos) throws IOException {
		if((pos < 0) || (pos >= blockLen)) { throw new IOException(); }
		curOffset = pos;
	}
	public void setLength(long newLength) throws IOException { throw new IOException("Bad Operation on Block"); }
	public void close() { this.file = null;	}

}
